@@ -2,7 +2,7 @@
  *
  *  littler - Provides hash-bang (#!) capability for R (www.r-project.org)
  *
- *  Copyright (C) 2006, 2007 Jeffrey Horner and Dirk Eddelbuettel
+ *  Copyright (C) 2006, 2007, 2008 Jeffrey Horner and Dirk Eddelbuettel
  *
  *  littler is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  $Id: littler.c 116 2007-02-19 20:43:16Z jeffrey.horner $
+ *  $Id: littler.c 136 2008-01-15 00:13:52Z dirk.eddelbuettel $
  */
 
 #include <getopt.h>
@@ -88,7 +88,7 @@ int source(char *file){
 	SET_STRING_ELT(f, 0, COPY_TO_USER_STRING(file));
 
 	/* Make print.eval argument */
-    PROTECT(p = NEW_LOGICAL(1));
+	PROTECT(p = NEW_LOGICAL(1));
 	LOGICAL_DATA(p)[0] = (verbose)? TRUE : FALSE;
 
 	/* expression source(f,print.eval=p) */
@@ -352,11 +352,11 @@ void littler_InitTempDir()
 	tmp = getenv("TMPDIR");
 	if (tmp == NULL) {
 		tmp = getenv("TMP");
-			if (tmp == NULL) { 
-				tmp = getenv("TEMP");
-				if (tmp == NULL) 
-					tmp = "/tmp";
-			}
+		if (tmp == NULL) { 
+			tmp = getenv("TEMP");
+			if (tmp == NULL) 
+				tmp = "/tmp";
+		}
 	}
 
 	R_TempDir=tmp;
@@ -387,6 +387,7 @@ void showHelpAndExit() {
 	       "  -V, --version        Show the version number\n"
 	       "  -v, --vanilla        Pass the '--vanilla' option to R\n"  
 	       "  -p, --verbose        Print the value of expressions to the console\n"  
+	       "  -l, --packages list  Load the R packages from the comma-separated 'list'\n"	
 	       "  -e, --eval  expr     Let R evaluate 'expr'\n"
 	       "\n\n",
 	       binaryName);
@@ -412,7 +413,7 @@ void showVersionAndExit() {
 					R_SVN_REVISION);
 		}
 	}
-  	printf("\n\nCopyright (C) 2006, 2007 Jeffrey Horner and Dirk Eddelbuettel\n"
+  	printf("\n\nCopyright (C) 2006, 2007, 2008 Jeffrey Horner and Dirk Eddelbuettel\n"
 	       "\n"
 	       "%s is free software and comes with ABSOLUTELY NO WARRANTY.\n"
 	       "You are welcome to redistribute it under the terms of the\n"
@@ -424,17 +425,20 @@ void showVersionAndExit() {
 
 void showUsageAndExit() {
   	printf("\n" 						
-	       "%s (aka '%s') can be used in three main modes.\n\n"
+	       "%s (aka '%s') can be used in four main modes.\n\n"
 	       "The first is via the so-called 'shebang' support it provides for GNU R.\n"
 	       "Suppose '%s' is installed in /usr/local/bin/%s. Then the first line of a\n"
 	       "script can be written as \"#!/usr/local/bin/%s\" and the rest of the file\n"
 	       "can contain standard R commands.  By setting executable permissions\n"
-	       "on the file, one can now create executable  R scripts.\n\n" 
+	       "on the file, one can now create executable R scripts.\n\n" 
 	       "The second is to supply a filename with commands that are to be\n"
 	       "evaluated.\n\n"
 	       "The third use is in standard compound command-line expressions common\n" 
 	       "under Unix (so called 'command pipes') as '%s' can take arguments\n"
 	       "from stdin if the special filename '-' is used to select stdin.\n\n"
+	       "The fourth use is in on-tge-fly evaluation of R expressions supplied\n"
+	       "via the -e or --eval options to provide a quick R expression tester\n"
+	       "and calculator.\n\n"
 	       "More documentation is provided in the '%s' manual page and via\n"
 	       "the tests directory in the sources.\n\n",
 	       binaryName, programName, binaryName, binaryName, binaryName,  programName, binaryName);
@@ -464,11 +468,12 @@ void init_rand()
 int main(int argc, char **argv){
 
 	/* R embedded arguments, and optional arguments to be picked via cmdline switches */
-	char *R_argv[] = {(char*)programName, "--gui=none", "--no-save", "--no-readline", "--silent", "", ""};
+	char *R_argv[] = {(char*)programName, "--gui=none", "--no-restore", "--no-save", "--no-readline", "--silent", "", ""};
 	char *R_argv_opt[] = {"--vanilla", "--slave"};
 	int R_argc = (sizeof(R_argv) - sizeof(R_argv_opt) ) / sizeof(R_argv[0]);
 	int i, nargv, c, optpos=0, vanilla=0;
 	char *evalstr = NULL;
+	char *libstr = NULL;
 	SEXP s_argv;
 
 	static struct option optargs[] = {
@@ -477,10 +482,11 @@ int main(int argc, char **argv){
 		{"version", no_argument,       NULL, 'V'},
 		{"vanilla", no_argument,       NULL, 'v'},
 		{"eval",    required_argument, NULL, 'e'},
+		{"packages",required_argument, NULL, 'l'},
 		{"verbose", no_argument,       NULL, 'p'},
 		{0, 0, 0, 0}
 	};
-	while ((c = getopt_long(argc, argv, "+hVve:np", optargs, &optpos)) != -1) {
+	while ((c = getopt_long(argc, argv, "+hVve:npl:", optargs, &optpos)) != -1) {
 		switch (c) {	
 			case 0:					/* numeric 0 is code for a long option */
 				/* printf ("Got option %s %d", optargs[optpos].name, optpos);*/
@@ -505,6 +511,9 @@ int main(int argc, char **argv){
 			case 'e':
 				evalstr = optarg;
 				break;
+			case 'l':
+				libstr = optarg;
+				break;
 			case 'v':	
 				vanilla=1;
 				break;
@@ -542,7 +551,7 @@ int main(int argc, char **argv){
 	 * So call stat(1) on it, and if its a file we will treat it as such.
 	 */
 	struct stat sbuf;
-	if (optind < argc) { 
+	if (optind < argc && evalstr==NULL) { 
 		if ((strcmp(argv[optind],"-") != 0) && (stat(argv[optind],&sbuf) != 0)) {
 			perror(argv[optind]);
 			exit(1);
@@ -586,18 +595,19 @@ int main(int argc, char **argv){
 	autoloads();
 
 	/* Place any argv arguments into argv vector in Global Environment */
-	if ((argc - optind) > 1){
+	/* if we have an evalstr supplied from -e|--eval, correct for it */
+	if ((argc - optind - (evalstr==NULL)) >= 1) {
+		int offset = (evalstr==NULL) + (strcmp(argv[optind],"-") == 0);
 		/* Build string vector */
-		nargv = argc - optind - 1;
+		nargv = argc - optind - offset;
 		PROTECT(s_argv = allocVector(STRSXP,nargv));
 		for (i = 0; i <nargv; i++){
-			STRING_PTR(s_argv)[i] = mkChar(argv[i+1+optind]);
+			STRING_PTR(s_argv)[i] = mkChar(argv[i+offset+optind]);
 			#ifdef DEBUG
-				printf("Passing %s to R\n", argv[i+1+optind]);
+				printf("Passing %s to R\n", argv[i+offset+optind]);
 			#endif
 		}
 		UNPROTECT(1);
-
 		setVar(install("argv"),s_argv,R_GlobalEnv);
 	} else {
 		setVar(install("argv"),R_NilValue,R_GlobalEnv);
@@ -606,8 +616,25 @@ int main(int argc, char **argv){
 	/* for tempfile() to work correctly */
 	init_rand();
 
-	/* Now determine which R code to evaluate */
+	/* deal with libraries, if requested */
+	if (libstr != NULL) {
+		char *ptr, *token, *strptr;
+		char buf[128];
 
+		ptr = token = libstr;
+		membuf_t pb = init_membuf(512);
+		while (token != NULL) {
+			token = strtok_r(ptr, ",", &strptr);
+			ptr = NULL; 			/* after initial call strtok expects NULL */
+			if (token != NULL) {
+				snprintf(buf, 127-27-strlen(token), "suppressMessages(library(%s));", token); 
+				parse_eval(&pb, buf, 1);
+			}
+		} 
+		destroy_membuf(pb);
+	}
+
+	/* Now determine which R code to evaluate */
 	int exit_val;
 	if (evalstr != NULL) {				
 		/* we have a command line expression to evaluate */
